@@ -1,16 +1,29 @@
-
 """Installs a WSGI application in place of a real host for testing.
 
 Introduction
 ============
 
-Testing a WSGI application normally involves starting a server at a
+Testing a WSGI application sometimes involves starting a server at a
 local host and port, then pointing your test code to that address.
 Instead, this library lets you intercept calls to any specific host/port
 combination and redirect them into a `WSGI application`_ importable by
 your test program. Thus, you can avoid spawning multiple processes or
 threads to test your Web app.
 
+Supported Libaries
+==================
+
+``wsgi_intercept`` works with a variety of HTTP clients in Python 2.7,
+3.3 and beyond, and in pypy.
+
+* urllib2
+* urllib.request
+* httplib
+* http.client
+* httplib2
+* requests
+* urllib3
+
 How Does It Work?
 =================
 
@@ -26,7 +39,7 @@ context manager over web requests that use the library associated with
 the subclass. For example::
 
     import httplib2
-    from wsgi_intercept.intercept import Httplib2Interceptor
+    from wsgi_intercept.interceptor import Httplib2Interceptor
     from mywsgiapp import app
 
     def load_app():
@@ -46,14 +59,21 @@ be used to specify which URLs should be redirected into what applications.
 These methods are still available, but the ``Interceptor`` classes are likely
 easier to use for most use cases.
 
-Note especially that ``app_create_fn`` is a *function object* returning a WSGI
-application; ``script_name`` becomes ``SCRIPT_NAME`` in the WSGI app's
-environment, if set.
+.. note:: ``app_create_fn`` is a *function object* returning a WSGI
+          application; ``script_name`` becomes ``SCRIPT_NAME`` in the WSGI
+          app's environment, if set.
+
+.. note:: If ``http_proxy`` or ``https_proxy`` is set in the environment
+          this can cause difficulties with some of the intercepted libraries.
+          If requests or urllib is being used, these will raise an exception
+          if one of those variables is set.
+
+.. note:: If ``wsgi_intercept.STRICT_RESPONSE_HEADERS`` is set to ``True``
+          then response headers sent by an application will be checked to
+          make sure they are of the type ``str`` native to the version of
+          Python, as required by pep 3333. The default is ``False`` (to
+          preserve backwards compatibility)
 
-Note also that if ``http_proxy`` or ``https_proxy`` is set in the environment
-this can cause difficulties with some of the intercepted libraries. If
-requests or urllib is being used, these will raise an exception if one of
-those variables is set.
 
 Install
 =======
@@ -65,17 +85,16 @@ Install
 Packages Intercepted
 ====================
 
-Unfortunately each of the Web testing frameworks uses its own specific
+Unfortunately each of the HTTP client libraries use their own specific
 mechanism for making HTTP call-outs, so individual implementations are
-needed. At this time there are implementations for ``httplib2`` and
-``requests`` in both Python 2 and 3, ``urllib2`` and ``httplib``
-in Python 2 and ``urllib.request`` and ``http.client`` in Python 3.
+needed. At this time there are implementations for ``httplib2``,
+``urllib3`` and ``requests`` in both Python 2 and 3, ``urllib2`` and
+``httplib`` in Python 2 and ``urllib.request`` and ``http.client``
+in Python 3.
 
 If you are using Python 2 and need support for a different HTTP
 client, require a version of ``wsgi_intercept<0.6``. Earlier versions
 include support for ``webtest``, ``webunit`` and ``zope.testbrowser``.
-It is quite likely that support for these versions will be relatively
-easy to add back in to the new version.
 
 The best way to figure out how to use interception is to inspect
 `the tests`_. More comprehensive documentation available upon
@@ -89,19 +108,21 @@ History
 
 Pursuant to Ian Bicking's `"best Web testing framework"`_ post, Titus
 Brown put together an `in-process HTTP-to-WSGI interception mechanism`_
-for his own Web testing system, twill_. Because the mechanism is pretty
+for his own Web testing system, twill. Because the mechanism is pretty
 generic -- it works at the httplib level -- Titus decided to try adding
 it into all of the *other* Python Web testing frameworks.
 
 The Python 2 version of wsgi-intercept was the result. Kumar McMillan
 later took over maintenance.
 
-The current version works with Python 2.7, 3.3, 3.4 and 3.5 and was assembled
-by `Chris Dent`_. Testing and documentation improvements from `Sasha Hart`_.
+The current version is tested with Python 2.7, 3.3, 3.4, 3.5, 3.6, and pypy
+and was assembled by `Chris Dent`_. Testing and documentation improvements
+from `Sasha Hart`_.
 
-.. _twill: http://www.idyll.org/~t/www-tools/twill.html
-.. _"best Web testing framework": http://blog.ianbicking.org/best-of-the-web-app-test-frameworks.html
-.. _in-process HTTP-to-WSGI interception mechanism: http://www.advogato.org/person/titus/diary.html?start=119
+.. _"best Web testing framework":
+     http://blog.ianbicking.org/best-of-the-web-app-test-frameworks.html
+.. _in-process HTTP-to-WSGI interception mechanism:
+     http://www.advogato.org/person/titus/diary.html?start=119
 .. _WSGI application: http://www.python.org/peps/pep-3333.html
 .. _Chris Dent: https://github.com/cdent
 .. _Sasha Hart: https://github.com/sashahart
@@ -116,30 +137,28 @@ Additional documentation is available on `Read The Docs`_.
 
 .. _GitHub: http://github.com/cdent/wsgi-intercept
 .. _Read The Docs: http://wsgi-intercept.readthedocs.org/en/latest/
-
 """
 from __future__ import print_function
 
-__version__ = '1.1.2'
-
-
 import sys
-try:
-    from http.client import HTTPConnection, HTTPSConnection
-except ImportError:
-    from httplib import HTTPConnection, HTTPSConnection
-
-try:
-    from io import BytesIO
-except ImportError:
-    from StringIO import StringIO as BytesIO
+import traceback
+from io import BytesIO
 
+# Don't use six here because it is unquote_to_bytes that we want in
+# Python 3.
 try:
     from urllib.parse import unquote_to_bytes as url_unquote
 except ImportError:
     from urllib import unquote as url_unquote
 
-import traceback
+import six
+from six.moves.http_client import HTTPConnection, HTTPSConnection
+
+
+# Set this to True to cause response headers from the intercepted
+# app to be confirmed as bytestrings, behaving as some wsgi servers.
+STRICT_RESPONSE_HEADERS = False
+
 
 debuglevel = 0
 # 1 basic
@@ -207,7 +226,7 @@ def make_environ(inp, host, port, script_name):
     environ = {}
 
     method_line = inp.readline()
-    if sys.version_info[0] > 2:
+    if six.PY3:
         method_line = method_line.decode('ISO-8859-1')
 
     content_type = None
@@ -220,7 +239,11 @@ def make_environ(inp, host, port, script_name):
 
         k, v = line.strip().split(b':', 1)
         v = v.lstrip()
-        v = v.decode('ISO-8859-1')
+        # Make header value a "native" string. PEP 3333 requires that
+        # string-like things in headers be of type `str`. Much of the
+        # time this isn't a problem but the SimpleCookie library does
+        # type checking against `type("")`.
+        v = str(v.decode('ISO-8859-1'))
 
         #
         # take care of special headers, and for the rest, put them
@@ -283,7 +306,7 @@ def make_environ(inp, host, port, script_name):
     # do to be like a server. Later various libraries will be forced
     # to decode and then reencode to get the UTF-8 that everyone
     # wants.
-    if sys.version_info[0] > 2:
+    if six.PY3:
         path_info = path_info.decode('latin-1')
 
     environ.update({
@@ -404,7 +427,8 @@ class wsgi_fake_socket:
 
         def start_response(status, headers, exc_info=None):
             # construct the HTTP request.
-            self.output.write(b"HTTP/1.0 " + status.encode('utf-8') + b"\n")
+            self.output.write(
+                b"HTTP/1.0 " + status.encode('ISO-8859-1') + b"\n")
             # Keep the reference of the headers list to write them only
             # when the whole application have been processed
             self.headers = headers
@@ -429,12 +453,17 @@ class wsgi_fake_socket:
         # send the headers
 
         for k, v in self.headers:
+            if STRICT_RESPONSE_HEADERS:
+                if not (isinstance(k, str) and isinstance(v, str)):
+                    raise TypeError(
+                        "Header has a key '%s' or value '%s' "
+                        "which is not a native str." % (k, v))
             try:
-                k = k.encode('utf-8')
+                k = k.encode('ISO-8859-1')
             except AttributeError:
                 pass
             try:
-                v = v.encode('utf-8')
+                v = v.encode('ISO-8859-1')
             except AttributeError:
                 pass
             self.output.write(k + b': ' + v + b"\n")
@@ -458,7 +487,7 @@ class wsgi_fake_socket:
                 for data in self.write_results:
                     self.output.write(data)
 
-            if generator_data:
+            if generator_data is not None:
                 try:
                     self.output.write(generator_data)
                 except TypeError as exc:
